import pathlib
import datetime
import os
import json
import re
from typing import (
    IO,
    Any,
    Callable,
    DefaultDict,
    Dict,
    Iterator,
    Optional,
    Type,
    TypeVar,
)


import datamodel_code_generator
import datamodel_code_generator.format
import datamodel_code_generator.imports

from datamodel_code_generator.parser.jsonschema import JsonSchemaParser
from datamodel_code_generator.parser.openapi import OpenAPIParser
from datamodel_code_generator import (
    DEFAULT_BASE_CLASS,
    Error,
    InputFileType,
    enable_debug_message,
)

from datamodel_code_generator.model.pydantic import (
    BaseModel,
    CustomRootType,
    DataModelField,
    dump_resolve_reference_action,
)


_underscorer1 = re.compile(r"(.)([A-Z][a-z]+)")
_underscorer2 = re.compile("([a-z0-9])([A-Z])")


def camel_to_snake(string):
    subbed = _underscorer1.sub(r"\1_\2", string)
    return _underscorer2.sub(r"\1_\2", subbed).lower()


VERSION = re.compile("v[0-9]+[a-z0-9]*$")


def make_named_class(model: BaseModel):
    model.base_class = "pdk8s.model.NamedModel"
    model.BASE_CLASS = "pdk8s.model.NamedModel"
    cls_import = datamodel_code_generator.imports.Import(import_="pdk8s.model")
    model.imports.append(cls_import)


def snakify_field(field: datamodel_code_generator.model.pydantic.DataModelField):
    original_name = field.name
    field.name = camel_to_snake(original_name)
    if field.name != original_name:
        field.alias = original_name


def remove_int_or_str(
    model, field: datamodel_code_generator.model.pydantic.DataModelField
):
    # the type_hint is cached and not recalculated - in the case
    # luckily as the data structure does not support the needed type hint
    # see also https://github.com/koxudaxi/datamodel-code-generator/issues/142

    union_import = datamodel_code_generator.imports.Import(
        from_="typing", import_="Union"
    )
    optional_import = datamodel_code_generator.imports.Import(
        from_="typing", import_="Optional"
    )
    if field.required:
        field.type_hint = "Union[int, str]"
        field.imports = [union_import]
    else:
        field.type_hint = "Optional[Union[int, str]]"
        field.imports = [union_import, optional_import]

    # cleanup model imports
    to_remove = []
    for import_ in model.imports:
        if "IntOrString" in import_.import_:
            to_remove.append(import_)

    for import_ in to_remove:
        model.imports.remove(import_)

    model.imports.extend(field.imports)


class K8SParser(JsonSchemaParser):
    def parse_raw(self):
        super(K8SParser, self).parse_raw()

        # self._classes_by_name = {}
        # self._single_value_enum = {}
        # for result in self.results:
        #     if result.base_class == "Enum":
        #         if len(result.fields) == 1:
        #             name = result.class_name
        #             assert name not in self._single_value_enum
        #             self._single_value_enum[name] = result.fields[0].name
        #         else:
        #             breakpoint
        # name = result.class_name
        # assert name not in self._classes_by_name
        # self._classes_by_name[name] = result

        new_results = []
        for result in self.results:
            self._mutate_class(result)
            new_results.append(result)

        self.results = new_results

    # top level objects:
    # x-kubernetes-group-version-kind

    def _mutate_class(self, model: datamodel_code_generator.model.DataModel):
        if model.class_name == "Model":
            return

        if len(model.fields) == 1 and model.fields[0].name is None:
            # defintion of a basic type, skip
            return

        if model.class_name == "IntOrString":
            # TODO
            return

        extra = '"forbid"'

        for field in model.fields:
            if field.name == "metadata":
                # For named classes we allow extra fields
                # so we have a nicer API with cls(name="Foo")
                # even with no actual field "name" existing.
                extra = '"allow"'
                make_named_class(model)

            if (
                len(field.data_types) == 1
                and field.data_types[0].type
                == "io.k8s.apimachinery.pkg.util.intstr.IntOrString"
            ):
                remove_int_or_str(model, field)

            snakify_field(field)

        model.extra_template_data["config"] = {"extra": extra}


def update_field(field):
    field_arguments = [
        f"{k}={field.get_valid_argument(v)}"
        for k, v in field.dict(include=field._FIELDS_KEYS).items()
        if v is not None
    ]
    field.field = f'Field({"..." if field.required else field.get_valid_argument(field.default)}, {",".join(field_arguments)})'


def add_init_py(path):
    for dirname, sub_dirs, filenames in os.walk(path):
        if "./" in dirname:
            # ignore hidden directories
            continue

        if "__init__.py" in filenames:
            continue

        open(os.path.join(dirname, "__init__.py"), "w").close()


def generate_models(output, input_text) -> None:
    """based on datamodel_code_generator.generate"""
    py37 = datamodel_code_generator.format.PythonVersion.PY_37
    target_python_version = py37

    validation = True
    base_class = "pydantic.BaseModel"
    custom_template_dir = None  # TODO seems not tot work, see https://github.com/koxudaxi/datamodel-code-generator/issues/144
    extra_template_data = None

    parser_class = K8SParser

    parser = parser_class(
        BaseModel,
        CustomRootType,
        DataModelField,
        base_class=base_class,
        custom_template_dir=custom_template_dir,
        extra_template_data=extra_template_data,
        target_python_version=target_python_version,
        text=input_text,
        dump_resolve_reference_action=dump_resolve_reference_action,
        validation=validation,
    )

    with datamodel_code_generator.chdir(output):
        result = parser.parse()

    if isinstance(result, str):
        modules = {output: result}
    else:
        if output is None:
            raise Error("Modular references require an output directory")
        if output.suffix:
            raise Error("Modular references require an output directory, not a file")
        modules = {
            output.joinpath(*name): body for name, body in sorted(result.items())
        }

    header = "# automatically generated file. DO NOT CHANGE MANUALLY"

    file: Optional[IO[Any]]
    for path, body in modules.items():
        if path is not None:
            if not path.parent.exists():
                path.parent.mkdir(parents=True)
            file = path.open("wt")
        else:
            file = None

        print(header, file=file)
        if body:
            print("", file=file)
            print(body.rstrip(), file=file)

        if file is not None:
            file.close()


def update_schema(schema: dict):
    """Adapt the kubernetes OpenAPI schema to our needs
    """
    definitions = schema["definitions"]
    for name, definition in definitions.items():
        version = None
        if "x-kubernetes-group-version-kind" in definition:
            k8s_versions = definition["x-kubernetes-group-version-kind"]
            if len(k8s_versions) != 1:
                print("warning, not correctly handling", name)

            k8s_version = k8s_versions[0]
            version = os.path.join(k8s_version["group"], k8s_version["version"])

        if "properties" in definition:
            # remove all fields named status
            # this might have some false positives, lets see what goes missing :)
            if "status" in definition["properties"]:
                definition["properties"].pop("status")

            for prop_name, prop in definition["properties"].items():
                # set the default of properties that are enums
                # with only one possible value to that value
                # This sets the `kind` property on all objects.
                if "enum" in prop and len(prop["enum"]) == 1:
                    prop.setdefault("default", prop["enum"][0])

                # set api version
                if prop_name == "apiVersion":
                    if version:
                        prop.setdefault("default", version)

                # inline IntOrString, avoids creation of IntOrString class
                # this allows assigning int or string directly, instead of this
                # ref_int_or_string = "#/definitions/io.k8s.apimachinery.pkg.util.intstr.IntOrString"
                # TODO using "oneOf" in properties is not supported by datamode-code-generator
                #      see https://github.com/koxudaxi/datamodel-code-generator/issues/139
                # if "$ref" in prop:
                #     ref = prop["$ref"]

                #     if ref == ref_int_or_string:
                #         prop.pop("$ref")

                #         prop.update(definitions[ref.split('/')[-1]])

                #         print(prop)
                # "$ref":
                #   "oneOf": [
                #     {
                #       "type": "string"
                #     },
                #     {
                #       "type": "integer"
                #     }
                #   ]


def download():
    tag = "v1.16.4"
    url = f"https://raw.githubusercontent.com/kubernetes/kubernetes/{tag}/api/openapi-spec/swagger.json"


def main():
    input_name = "k8s_1.16.4.json"
    schema = json.load(open(input_name))

    update_schema(schema)

    output = pathlib.Path(__file__).parent / pathlib.Path("gen")
    generate_models(output, json.dumps(schema))
    add_init_py(output)
    # run_mypy()


if __name__ == "__main__":
    main()



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind39, Kind40
from ...apimachinery.pkg.apis.meta import v1


class BoundObjectReference(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        None, alias="apiVersion", description="API version of the referent."
    )
    kind: Optional[str] = Field(
        None, description="Kind of the referent. Valid kinds are 'Pod' and 'Secret'."
    )
    name: Optional[str] = Field(None, description="Name of the referent.")
    uid: Optional[str] = Field(None, description="UID of the referent.")


class TokenRequestSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    audiences: List[str] = Field(
        ...,
        description="Audiences are the intendend audiences of the token. A recipient of a token must identitfy themself with an identifier in the list of audiences of the token, and otherwise should reject the token. A token issued for multiple audiences may be used to authenticate against any of the audiences listed but implies a high degree of trust between the target audiences.",
    )
    bound_object_ref: Optional[BoundObjectReference] = Field(
        None,
        alias="boundObjectRef",
        description="BoundObjectRef is a reference to an object that the token will be bound to. The token will only be valid for as long as the bound object exists. NOTE: The API server's TokenReview endpoint will validate the BoundObjectRef, but other audiences may not. Keep ExpirationSeconds small if you want prompt revocation.",
    )
    expiration_seconds: Optional[int] = Field(
        None,
        alias="expirationSeconds",
        description="ExpirationSeconds is the requested duration of validity of the request. The token issuer may return a token with a different validity duration so a client needs to check the 'expiration' field in a response.",
    )


class TokenReviewSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    audiences: Optional[List[str]] = Field(
        None,
        description="Audiences is a list of the identifiers that the resource server presented with the token identifies as. Audience-aware token authenticators will verify that the token was intended for at least one of the audiences in this list. If no audiences are provided, the audience will default to the audience of the Kubernetes apiserver.",
    )
    token: Optional[str] = Field(None, description="Token is the opaque bearer token.")


class UserInfo(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    extra: Optional[Dict[str, Any]] = Field(
        None, description="Any additional information provided by the authenticator."
    )
    groups: Optional[List[str]] = Field(
        None, description="The names of groups this user is a part of."
    )
    uid: Optional[str] = Field(
        None,
        description="A unique value that identifies this user across time. If this user is deleted and another user by the same name is added, they will have different UIDs.",
    )
    username: Optional[str] = Field(
        None,
        description="The name that uniquely identifies this user among all active users.",
    )


class TokenRequestStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    expiration_timestamp: v1.Time = Field(
        ...,
        alias="expirationTimestamp",
        description="ExpirationTimestamp is the time of expiration of the returned token.",
    )
    token: str = Field(..., description="Token is the opaque bearer token.")


class TokenReviewStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    audiences: Optional[List[str]] = Field(
        None,
        description='Audiences are audience identifiers chosen by the authenticator that are compatible with both the TokenReview and token. An identifier is any identifier in the intersection of the TokenReviewSpec audiences and the token\'s audiences. A client of the TokenReview API that sets the spec.audiences field should validate that a compatible audience identifier is returned in the status.audiences field to ensure that the TokenReview server is audience aware. If a TokenReview returns an empty status.audience field where status.authenticated is "true", the token is valid against the audience of the Kubernetes API server.',
    )
    authenticated: Optional[bool] = Field(
        None,
        description="Authenticated indicates that the token was associated with a known user.",
    )
    error: Optional[str] = Field(
        None, description="Error indicates that the token couldn't be checked"
    )
    user: Optional[UserInfo] = Field(
        None, description="User is the UserInfo associated with the provided token."
    )


class TokenRequest(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "authentication.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind39] = Field(
        "TokenRequest",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: TokenRequestSpec


class TokenReview(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "authentication.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind40] = Field(
        "TokenReview",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: TokenReviewSpec = Field(
        ..., description="Spec holds information about the request being evaluated"
    )



import os
import collections.abc

import click
import yaml


def cleanup(data):
    if isinstance(data, dict):
        return cleanup_dict(data)
    elif isinstance(data, list):
        return [cleanup(item) for item in data]
    else:
        return data


def cleanup_dict(data):
    """remove all keys with value None
    """
    result = {}
    for key, value in data.items():
        if value is None:
            continue

        elif isinstance(value, dict):
            result[key] = cleanup_dict(value)

        else:
            result[key] = cleanup(value)

    return result


def definition_to_string(k8s_obj) -> str:
    if hasattr(k8s_obj, "dict"):
        k8s_dict = k8s_obj.dict(by_alias=True)
    elif isinstance(k8s_obj, dict):
        k8s_dict = k8s_obj
    else:
        raise AttributeError("cannot handle k8s_obj " + repr(k8s_obj))

    k8s_def = cleanup_dict(k8s_dict)
    return yaml.safe_dump(k8s_def)


def chart_to_string(chart) -> str:
    return "---\n".join(map(definition_to_string, chart))


def synth(chart_variables: dict, output_format: str, output: str) -> int:
    if not "chart" in chart_variables:
        click.secho(f'{input} did not define a variable: "chart"', fg="red", bold=True)
        return 1
    
    chart = chart_variables["chart"]

    if not isinstance(chart, collections.abc.Iterable):
        click.secho(f'{input}:chart must be iterable, e.g. a list or k8s.Chart', fg="red", bold=True)
        return 2

    if not os.path.exists(output):
        os.makedirs(output)

    chart_text = chart_to_string(chart)
    header = "# This file is generated by pdk8s, editing is strongly discouraged\n"
    description = chart_variables.get("__doc__", "")
    description = description.strip()

    if output_format == "helm":
        helm_chart = {
            "apiVersion": "v2",
            "name": chart_variables["name"],
            "description": description,
            "type": "application",
            "version": chart_variables["chart_version"],
            "appVersion": chart_variables["app_version"]
        }

        with open(os.path.join(output, "Chart.yaml"), "w") as f:
            f.write(header)
            f.write(yaml.safe_dump(helm_chart))

        with open(os.path.join(output, "values.yaml"), "w") as f:
            f.write(header)
            f.write("# This helm chart does not support values via helm.")
            f.write("# To parameterize it, use the pdk8s source of this chart")

        templates_path = os.path.join(output, "templates")
        if not os.path.exists(templates_path):
            os.makedirs(templates_path)

        with open(os.path.join(templates_path, "generated.yaml"), "w") as f:
            f.write(header)
            f.write(chart_text)

    elif output_format == "kubernetes":
        with open(os.path.join(output, "chart.yaml")) as f:
            f.write(header)
            for line in description.split('\n'):
                f.write(f'# {line}\n')
            f.write(chart_text)

    else:
        raise NotImplementedError()

    return 0



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional, Union

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import (
    Kind26,
    Kind27,
    Kind28,
    Kind29,
    Kind30,
    Kind31,
    Kind32,
    Kind33,
    Kind34,
    Kind35,
    Kind36,
)
from ...apimachinery.pkg import runtime
from ...apimachinery.pkg.apis.meta import v1
from ...apimachinery.pkg.util import intstr
from ..core import v1 as v1_1


class RollingUpdateStatefulSetStrategy(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    partition: Optional[int] = Field(
        None,
        description="Partition indicates the ordinal at which the StatefulSet should be partitioned. Default value is 0.",
    )


class ScaleSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    replicas: Optional[int] = Field(
        None, description="desired number of instances for the scaled object."
    )


class ScaleStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    replicas: int = Field(
        ..., description="actual number of observed instances of the scaled object."
    )
    selector: Optional[Dict[str, Any]] = Field(
        None,
        description="label query over pods that should match the replicas count. More info: http://kubernetes.io/docs/user-guide/labels#label-selectors",
    )
    target_selector: Optional[str] = Field(
        None,
        alias="targetSelector",
        description="label selector for pods that should match the replicas count. This is a serializated version of both map-based and more expressive set-based selectors. This is done to avoid introspection in the clients. The string will be in the same format as the query-param syntax. If the target type only supports map-based selectors, both this field and map-based selector field are populated. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )


class StatefulSetUpdateStrategy(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    rolling_update: Optional[RollingUpdateStatefulSetStrategy] = Field(
        None,
        alias="rollingUpdate",
        description="RollingUpdate is used to communicate parameters when Type is RollingUpdateStatefulSetStrategyType.",
    )
    type: Optional[str] = Field(
        None,
        description="Type indicates the type of the StatefulSetUpdateStrategy. Default is RollingUpdate.",
    )


class DaemonSetCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="Last time the condition transitioned from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="A human readable message indicating details about the transition.",
    )
    reason: Optional[str] = Field(
        None, description="The reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of DaemonSet condition.")


class DaemonSetStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    collision_count: Optional[int] = Field(
        None,
        alias="collisionCount",
        description="Count of hash collisions for the DaemonSet. The DaemonSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
    )
    conditions: Optional[List[DaemonSetCondition]] = Field(
        None,
        description="Represents the latest available observations of a DaemonSet's current state.",
    )
    current_number_scheduled: int = Field(
        ...,
        alias="currentNumberScheduled",
        description="The number of nodes that are running at least 1 daemon pod and are supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
    )
    desired_number_scheduled: int = Field(
        ...,
        alias="desiredNumberScheduled",
        description="The total number of nodes that should be running the daemon pod (including nodes correctly running the daemon pod). More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
    )
    number_available: Optional[int] = Field(
        None,
        alias="numberAvailable",
        description="The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and available (ready for at least spec.minReadySeconds)",
    )
    number_misscheduled: int = Field(
        ...,
        alias="numberMisscheduled",
        description="The number of nodes that are running the daemon pod, but are not supposed to run the daemon pod. More info: https://kubernetes.io/docs/concepts/workloads/controllers/daemonset/",
    )
    number_ready: int = Field(
        ...,
        alias="numberReady",
        description="The number of nodes that should be running the daemon pod and have one or more of the daemon pod running and ready.",
    )
    number_unavailable: Optional[int] = Field(
        None,
        alias="numberUnavailable",
        description="The number of nodes that should be running the daemon pod and have none of the daemon pod running and available (ready for at least spec.minReadySeconds)",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="The most recent generation observed by the daemon set controller.",
    )
    updated_number_scheduled: Optional[int] = Field(
        None,
        alias="updatedNumberScheduled",
        description="The total number of nodes that are running updated daemon pod",
    )


class DeploymentCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="Last time the condition transitioned from one status to another.",
    )
    last_update_time: Optional[v1.Time] = Field(
        None,
        alias="lastUpdateTime",
        description="The last time this condition was updated.",
    )
    message: Optional[str] = Field(
        None,
        description="A human readable message indicating details about the transition.",
    )
    reason: Optional[str] = Field(
        None, description="The reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of deployment condition.")


class DeploymentStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    available_replicas: Optional[int] = Field(
        None,
        alias="availableReplicas",
        description="Total number of available pods (ready for at least minReadySeconds) targeted by this deployment.",
    )
    collision_count: Optional[int] = Field(
        None,
        alias="collisionCount",
        description="Count of hash collisions for the Deployment. The Deployment controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ReplicaSet.",
    )
    conditions: Optional[List[DeploymentCondition]] = Field(
        None,
        description="Represents the latest available observations of a deployment's current state.",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="The generation observed by the deployment controller.",
    )
    ready_replicas: Optional[int] = Field(
        None,
        alias="readyReplicas",
        description="Total number of ready pods targeted by this deployment.",
    )
    replicas: Optional[int] = Field(
        None,
        description="Total number of non-terminated pods targeted by this deployment (their labels match the selector).",
    )
    unavailable_replicas: Optional[int] = Field(
        None,
        alias="unavailableReplicas",
        description="Total number of unavailable pods targeted by this deployment. This is the total number of pods that are still required for the deployment to have 100% available capacity. They may either be pods that are running but not yet available or pods that still have not been created.",
    )
    updated_replicas: Optional[int] = Field(
        None,
        alias="updatedReplicas",
        description="Total number of non-terminated pods targeted by this deployment that have the desired template spec.",
    )


class ReplicaSetCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="The last time the condition transitioned from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="A human readable message indicating details about the transition.",
    )
    reason: Optional[str] = Field(
        None, description="The reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of replica set condition.")


class ReplicaSetStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    available_replicas: Optional[int] = Field(
        None,
        alias="availableReplicas",
        description="The number of available replicas (ready for at least minReadySeconds) for this replica set.",
    )
    conditions: Optional[List[ReplicaSetCondition]] = Field(
        None,
        description="Represents the latest available observations of a replica set's current state.",
    )
    fully_labeled_replicas: Optional[int] = Field(
        None,
        alias="fullyLabeledReplicas",
        description="The number of pods that have labels matching the labels of the pod template of the replicaset.",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="ObservedGeneration reflects the generation of the most recently observed ReplicaSet.",
    )
    ready_replicas: Optional[int] = Field(
        None,
        alias="readyReplicas",
        description="The number of ready replicas for this replica set.",
    )
    replicas: int = Field(
        ...,
        description="Replicas is the most recently oberved number of replicas. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
    )


class RollingUpdateDaemonSet(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max_unavailable: Optional[Union[int, str]] = Field(
        None,
        alias="maxUnavailable",
        description="The maximum number of DaemonSet pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of total number of DaemonSet pods at the start of the update (ex: 10%). Absolute number is calculated from percentage by rounding up. This cannot be 0. Default value is 1. Example: when this is set to 30%, at most 30% of the total number of nodes that should be running the daemon pod (i.e. status.desiredNumberScheduled) can have their pods stopped for an update at any given time. The update starts by stopping at most 30% of those DaemonSet pods and then brings up new DaemonSet pods in their place. Once the new pods are available, it then proceeds onto other DaemonSet pods, thus ensuring that at least 70% of original number of DaemonSet pods are available at all times during the update.",
    )


class RollingUpdateDeployment(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max_surge: Optional[Union[int, str]] = Field(
        None,
        alias="maxSurge",
        description="The maximum number of pods that can be scheduled above the desired number of pods. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). This can not be 0 if MaxUnavailable is 0. Absolute number is calculated from percentage by rounding up. Defaults to 25%. Example: when this is set to 30%, the new ReplicaSet can be scaled up immediately when the rolling update starts, such that the total number of old and new pods do not exceed 130% of desired pods. Once old pods have been killed, new ReplicaSet can be scaled up further, ensuring that total number of pods running at any time during the update is at most 130% of desired pods.",
    )
    max_unavailable: Optional[Union[int, str]] = Field(
        None,
        alias="maxUnavailable",
        description="The maximum number of pods that can be unavailable during the update. Value can be an absolute number (ex: 5) or a percentage of desired pods (ex: 10%). Absolute number is calculated from percentage by rounding down. This can not be 0 if MaxSurge is 0. Defaults to 25%. Example: when this is set to 30%, the old ReplicaSet can be scaled down to 70% of desired pods immediately when the rolling update starts. Once new pods are ready, old ReplicaSet can be scaled down further, followed by scaling up the new ReplicaSet, ensuring that the total number of pods available at all times during the update is at least 70% of desired pods.",
    )


class StatefulSetCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="Last time the condition transitioned from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="A human readable message indicating details about the transition.",
    )
    reason: Optional[str] = Field(
        None, description="The reason for the condition's last transition."
    )
    type: str = Field(..., description="Type of statefulset condition.")


class StatefulSetStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    collision_count: Optional[int] = Field(
        None,
        alias="collisionCount",
        description="collisionCount is the count of hash collisions for the StatefulSet. The StatefulSet controller uses this field as a collision avoidance mechanism when it needs to create the name for the newest ControllerRevision.",
    )
    conditions: Optional[List[StatefulSetCondition]] = Field(
        None,
        description="Represents the latest available observations of a statefulset's current state.",
    )
    current_replicas: Optional[int] = Field(
        None,
        alias="currentReplicas",
        description="currentReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by currentRevision.",
    )
    current_revision: Optional[str] = Field(
        None,
        alias="currentRevision",
        description="currentRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [0,currentReplicas).",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="observedGeneration is the most recent generation observed for this StatefulSet. It corresponds to the StatefulSet's generation, which is updated on mutation by the API Server.",
    )
    ready_replicas: Optional[int] = Field(
        None,
        alias="readyReplicas",
        description="readyReplicas is the number of Pods created by the StatefulSet controller that have a Ready Condition.",
    )
    replicas: int = Field(
        ...,
        description="replicas is the number of Pods created by the StatefulSet controller.",
    )
    update_revision: Optional[str] = Field(
        None,
        alias="updateRevision",
        description="updateRevision, if not empty, indicates the version of the StatefulSet used to generate Pods in the sequence [replicas-updatedReplicas,replicas)",
    )
    updated_replicas: Optional[int] = Field(
        None,
        alias="updatedReplicas",
        description="updatedReplicas is the number of Pods created by the StatefulSet controller from the StatefulSet version indicated by updateRevision.",
    )


class ControllerRevision(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    data: Optional[runtime.RawExtension] = Field(
        None, description="Data is the serialized representation of the state."
    )
    kind: Optional[Kind26] = Field(
        "ControllerRevision",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    revision: int = Field(
        ...,
        description="Revision indicates the revision of the state represented by Data.",
    )


class ControllerRevisionList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[ControllerRevision] = Field(
        ..., description="Items is the list of ControllerRevisions"
    )
    kind: Optional[Kind27] = Field(
        "ControllerRevisionList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class DaemonSetUpdateStrategy(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    rolling_update: Optional[RollingUpdateDaemonSet] = Field(
        None,
        alias="rollingUpdate",
        description='Rolling update config params. Present only if type = "RollingUpdate".',
    )
    type: Optional[str] = Field(
        None,
        description='Type of daemon set update. Can be "RollingUpdate" or "OnDelete". Default is RollingUpdate.',
    )


class DeploymentStrategy(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    rolling_update: Optional[RollingUpdateDeployment] = Field(
        None,
        alias="rollingUpdate",
        description="Rolling update config params. Present only if DeploymentStrategyType = RollingUpdate.",
    )
    type: Optional[str] = Field(
        None,
        description='Type of deployment. Can be "Recreate" or "RollingUpdate". Default is RollingUpdate.',
    )


class Scale(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind34] = Field(
        "Scale",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.",
    )
    spec: Optional[ScaleSpec] = Field(
        None,
        description="defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.",
    )


class DaemonSetSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    min_ready_seconds: Optional[int] = Field(
        None,
        alias="minReadySeconds",
        description="The minimum number of seconds for which a newly created DaemonSet pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready).",
    )
    revision_history_limit: Optional[int] = Field(
        None,
        alias="revisionHistoryLimit",
        description="The number of old history to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
    )
    selector: v1.LabelSelector = Field(
        ...,
        description="A label query over pods that are managed by the daemon set. Must match in order to be controlled. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )
    template: v1_1.PodTemplateSpec = Field(
        ...,
        description="An object that describes the pod that will be created. The DaemonSet will create exactly one copy of this pod on every node that matches the template's node selector (or on every node if no node selector is specified). More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
    )
    update_strategy: Optional[DaemonSetUpdateStrategy] = Field(
        None,
        alias="updateStrategy",
        description="An update strategy to replace existing DaemonSet pods with new pods.",
    )


class DeploymentSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    min_ready_seconds: Optional[int] = Field(
        None,
        alias="minReadySeconds",
        description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
    )
    paused: Optional[bool] = Field(
        None, description="Indicates that the deployment is paused."
    )
    progress_deadline_seconds: Optional[int] = Field(
        None,
        alias="progressDeadlineSeconds",
        description="The maximum time in seconds for a deployment to make progress before it is considered to be failed. The deployment controller will continue to process failed deployments and a condition with a ProgressDeadlineExceeded reason will be surfaced in the deployment status. Note that progress will not be estimated during the time a deployment is paused. Defaults to 600s.",
    )
    replicas: Optional[int] = Field(
        None,
        description="Number of desired pods. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
    )
    revision_history_limit: Optional[int] = Field(
        None,
        alias="revisionHistoryLimit",
        description="The number of old ReplicaSets to retain to allow rollback. This is a pointer to distinguish between explicit zero and not specified. Defaults to 10.",
    )
    selector: v1.LabelSelector = Field(
        ...,
        description="Label selector for pods. Existing ReplicaSets whose pods are selected by this will be the ones affected by this deployment. It must match the pod template's labels.",
    )
    strategy: Optional[DeploymentStrategy] = Field(
        None,
        description="The deployment strategy to use to replace existing pods with new ones.",
    )
    template: v1_1.PodTemplateSpec = Field(
        ..., description="Template describes the pods that will be created."
    )


class ReplicaSetSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    min_ready_seconds: Optional[int] = Field(
        None,
        alias="minReadySeconds",
        description="Minimum number of seconds for which a newly created pod should be ready without any of its container crashing, for it to be considered available. Defaults to 0 (pod will be considered available as soon as it is ready)",
    )
    replicas: Optional[int] = Field(
        None,
        description="Replicas is the number of desired replicas. This is a pointer to distinguish between explicit zero and unspecified. Defaults to 1. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller/#what-is-a-replicationcontroller",
    )
    selector: v1.LabelSelector = Field(
        ...,
        description="Selector is a label query over pods that should match the replica count. Label keys and values that must match in order to be controlled by this replica set. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )
    template: Optional[v1_1.PodTemplateSpec] = Field(
        None,
        description="Template is the object that describes the pod that will be created if insufficient replicas are detected. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller#pod-template",
    )


class StatefulSetSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    pod_management_policy: Optional[str] = Field(
        None,
        alias="podManagementPolicy",
        description="podManagementPolicy controls how pods are created during initial scale up, when replacing pods on nodes, or when scaling down. The default policy is `OrderedReady`, where pods are created in increasing order (pod-0, then pod-1, etc) and the controller will wait until each pod is ready before continuing. When scaling down, the pods are removed in the opposite order. The alternative policy is `Parallel` which will create pods in parallel to match the desired scale without waiting, and on scale down will delete all pods at once.",
    )
    replicas: Optional[int] = Field(
        None,
        description="replicas is the desired number of replicas of the given Template. These are replicas in the sense that they are instantiations of the same Template, but individual replicas also have a consistent identity. If unspecified, defaults to 1.",
    )
    revision_history_limit: Optional[int] = Field(
        None,
        alias="revisionHistoryLimit",
        description="revisionHistoryLimit is the maximum number of revisions that will be maintained in the StatefulSet's revision history. The revision history consists of all revisions not represented by a currently applied StatefulSetSpec version. The default value is 10.",
    )
    selector: v1.LabelSelector = Field(
        ...,
        description="selector is a label query over pods that should match the replica count. It must match the pod template's labels. More info: https://kubernetes.io/docs/concepts/overview/working-with-objects/labels/#label-selectors",
    )
    service_name: str = Field(
        ...,
        alias="serviceName",
        description='serviceName is the name of the service that governs this StatefulSet. This service must exist before the StatefulSet, and is responsible for the network identity of the set. Pods get DNS/hostnames that follow the pattern: pod-specific-string.serviceName.default.svc.cluster.local where "pod-specific-string" is managed by the StatefulSet controller.',
    )
    template: v1_1.PodTemplateSpec = Field(
        ...,
        description="template is the object that describes the pod that will be created if insufficient replicas are detected. Each pod stamped out by the StatefulSet will fulfill this Template, but have a unique identity from the rest of the StatefulSet.",
    )
    update_strategy: Optional[StatefulSetUpdateStrategy] = Field(
        None,
        alias="updateStrategy",
        description="updateStrategy indicates the StatefulSetUpdateStrategy that will be employed to update Pods in the StatefulSet when a revision is made to Template.",
    )
    volume_claim_templates: Optional[List[v1_1.PersistentVolumeClaim]] = Field(
        None,
        alias="volumeClaimTemplates",
        description="volumeClaimTemplates is a list of claims that pods are allowed to reference. The StatefulSet controller is responsible for mapping network identities to claims in a way that maintains the identity of a pod. Every claim in this list must have at least one matching (by name) volumeMount in one container in the template. A claim in this list takes precedence over any volumes in the template, with the same name.",
    )


class DaemonSet(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind28] = Field(
        "DaemonSet",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[DaemonSetSpec] = Field(
        None,
        description="The desired behavior of this daemon set. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class DaemonSetList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[DaemonSet] = Field(..., description="A list of daemon sets.")
    kind: Optional[Kind29] = Field(
        "DaemonSetList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )


class Deployment(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind30] = Field(
        "Deployment",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None, description="Standard object metadata."
    )
    spec: Optional[DeploymentSpec] = Field(
        None, description="Specification of the desired behavior of the Deployment."
    )


class DeploymentList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[Deployment] = Field(
        ..., description="Items is the list of Deployments."
    )
    kind: Optional[Kind31] = Field(
        "DeploymentList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(None, description="Standard list metadata.")


class ReplicaSet(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind32] = Field(
        "ReplicaSet",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="If the Labels of a ReplicaSet are empty, they are defaulted to be the same as the Pod(s) that the ReplicaSet manages. Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[ReplicaSetSpec] = Field(
        None,
        description="Spec defines the specification of the desired behavior of the ReplicaSet. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class ReplicaSetList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[ReplicaSet] = Field(
        ...,
        description="List of ReplicaSets. More info: https://kubernetes.io/docs/concepts/workloads/controllers/replicationcontroller",
    )
    kind: Optional[Kind33] = Field(
        "ReplicaSetList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )


class StatefulSet(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind35] = Field(
        "StatefulSet",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: Optional[StatefulSetSpec] = Field(
        None, description="Spec defines the desired identities of pods in this set."
    )


class StatefulSetList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apps/v1beta2",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[StatefulSet]
    kind: Optional[Kind36] = Field(
        "StatefulSetList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = None



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind163, Kind164
from ...apimachinery.pkg.apis.meta import v1 as v1_1
from ..core import v1


class PodPresetSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    env: Optional[List[v1.EnvVar]] = Field(
        None,
        description="Env defines the collection of EnvVar to inject into containers.",
    )
    env_from: Optional[List[v1.EnvFromSource]] = Field(
        None,
        alias="envFrom",
        description="EnvFrom defines the collection of EnvFromSource to inject into containers.",
    )
    selector: Optional[v1_1.LabelSelector] = Field(
        None,
        description="Selector is a label query over a set of resources, in this case pods. Required.",
    )
    volume_mounts: Optional[List[v1.VolumeMount]] = Field(
        None,
        alias="volumeMounts",
        description="VolumeMounts defines the collection of VolumeMount to inject into containers.",
    )
    volumes: Optional[List[v1.Volume]] = Field(
        None,
        description="Volumes defines the collection of Volume to inject into the pod.",
    )


class PodPreset(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "settings.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind163] = Field(
        "PodPreset",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: Optional[PodPresetSpec] = None


class PodPresetList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "settings.k8s.io/v1alpha1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[PodPreset] = Field(
        ..., description="Items is a list of schema objects."
    )
    kind: Optional[Kind164] = Field(
        "PodPresetList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



from typing import Optional

import pydantic


def _set_name(metadata, value):
    if metadata is None:
        return {"name": value}

    if isinstance(metadata, dict):
        metadata["name"] = value
    else:
        metadata.name = value

    return metadata


class NamedModel(pydantic.BaseModel):
    def _get_name(self) -> Optional[str]:
        if not self.metadata:
            return None
        return self.metadata.name

    name = property(_get_name)

    def __setattr__(self, name, value):
        if name == "name":
            self.metadata = _set_name(self.metadata, value)
        else:
            super().__setattr__(name, value)

    @pydantic.root_validator(pre=True)
    def extract_name(cls, values):
        if "name" in values:
            values["metadata"] = _set_name(values.get("metadata"), values["name"])
            del values["name"]

        return values



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional, Union

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind120, Kind121
from ...apimachinery.pkg.apis.meta import v1
from ...apimachinery.pkg.util import intstr


class IPBlock(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    cidr: str = Field(
        ...,
        description='CIDR is a string representing the IP Block Valid examples are "192.168.1.1/24"',
    )
    except_: Optional[List[str]] = Field(
        None,
        alias="except",
        description='Except is a slice of CIDRs that should not be included within an IP Block Valid examples are "192.168.1.1/24" Except values will be rejected if they are outside the CIDR range',
    )


class NetworkPolicyPort(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    port: Optional[Union[int, str]] = Field(
        None,
        description="The port on the given protocol. This can either be a numerical or named port on a pod. If this field is not provided, this matches all port names and numbers.",
    )
    protocol: Optional[str] = Field(
        None,
        description="The protocol (TCP, UDP, or SCTP) which traffic must match. If not specified, this field defaults to TCP.",
    )


class NetworkPolicyPeer(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ip_block: Optional[IPBlock] = Field(
        None,
        alias="ipBlock",
        description="IPBlock defines policy on a particular IPBlock. If this field is set then neither of the other fields can be.",
    )
    namespace_selector: Optional[v1.LabelSelector] = Field(
        None,
        alias="namespaceSelector",
        description="Selects Namespaces using cluster-scoped labels. This field follows standard label selector semantics; if present but empty, it selects all namespaces.\n\nIf PodSelector is also set, then the NetworkPolicyPeer as a whole selects the Pods matching PodSelector in the Namespaces selected by NamespaceSelector. Otherwise it selects all Pods in the Namespaces selected by NamespaceSelector.",
    )
    pod_selector: Optional[v1.LabelSelector] = Field(
        None,
        alias="podSelector",
        description="This is a label selector which selects Pods. This field follows standard label selector semantics; if present but empty, it selects all pods.\n\nIf NamespaceSelector is also set, then the NetworkPolicyPeer as a whole selects the Pods matching PodSelector in the Namespaces selected by NamespaceSelector. Otherwise it selects the Pods matching PodSelector in the policy's own Namespace.",
    )


class NetworkPolicyEgressRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ports: Optional[List[NetworkPolicyPort]] = Field(
        None,
        description="List of destination ports for outgoing traffic. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
    )
    to: Optional[List[NetworkPolicyPeer]] = Field(
        None,
        description="List of destinations for outgoing traffic of pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all destinations (traffic not restricted by destination). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the to list.",
    )


class NetworkPolicyIngressRule(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    from_: Optional[List[NetworkPolicyPeer]] = Field(
        None,
        alias="from",
        description="List of sources which should be able to access the pods selected for this rule. Items in this list are combined using a logical OR operation. If this field is empty or missing, this rule matches all sources (traffic not restricted by source). If this field is present and contains at least one item, this rule allows traffic only if the traffic matches at least one item in the from list.",
    )
    ports: Optional[List[NetworkPolicyPort]] = Field(
        None,
        description="List of ports which should be made accessible on the pods selected for this rule. Each item in this list is combined using a logical OR. If this field is empty or missing, this rule matches all ports (traffic not restricted by port). If this field is present and contains at least one item, then this rule allows traffic only if the traffic matches at least one port in the list.",
    )


class NetworkPolicySpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    egress: Optional[List[NetworkPolicyEgressRule]] = Field(
        None,
        description="List of egress rules to be applied to the selected pods. Outgoing traffic is allowed if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic matches at least one egress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy limits all outgoing traffic (and serves solely to ensure that the pods it selects are isolated by default). This field is beta-level in 1.8",
    )
    ingress: Optional[List[NetworkPolicyIngressRule]] = Field(
        None,
        description="List of ingress rules to be applied to the selected pods. Traffic is allowed to a pod if there are no NetworkPolicies selecting the pod (and cluster policy otherwise allows the traffic), OR if the traffic source is the pod's local node, OR if the traffic matches at least one ingress rule across all of the NetworkPolicy objects whose podSelector matches the pod. If this field is empty then this NetworkPolicy does not allow any traffic (and serves solely to ensure that the pods it selects are isolated by default)",
    )
    pod_selector: v1.LabelSelector = Field(
        ...,
        alias="podSelector",
        description="Selects the pods to which this NetworkPolicy object applies. The array of ingress rules is applied to any pods selected by this field. Multiple network policies can select the same set of pods. In this case, the ingress rules for each are combined additively. This field is NOT optional and follows standard label selector semantics. An empty podSelector matches all pods in this namespace.",
    )
    policy_types: Optional[List[str]] = Field(
        None,
        alias="policyTypes",
        description='List of rule types that the NetworkPolicy relates to. Valid options are "Ingress", "Egress", or "Ingress,Egress". If this field is not specified, it will default based on the existence of Ingress or Egress rules; policies that contain an Egress section are assumed to affect Egress, and all policies (whether or not they contain an Ingress section) are assumed to affect Ingress. If you want to write an egress-only policy, you must explicitly specify policyTypes [ "Egress" ]. Likewise, if you want to write a policy that specifies that no egress is allowed, you must specify a policyTypes value that include "Egress" (since such a policy would not include an Egress section and would otherwise default to just [ "Ingress" ]). This field is beta-level in 1.8',
    )


class NetworkPolicy(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "networking.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind120] = Field(
        "NetworkPolicy",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[NetworkPolicySpec] = Field(
        None,
        description="Specification of the desired behavior for this NetworkPolicy.",
    )


class NetworkPolicyList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "networking.k8s.io/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[NetworkPolicy] = Field(
        ..., description="Items is a list of schema objects."
    )
    kind: Optional[Kind121] = Field(
        "NetworkPolicyList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from datetime import datetime
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ....... import Kind183, Kind184, Kind185, Kind186, Kind187, Kind188
from ... import runtime


class APIResource(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    categories: Optional[List[str]] = Field(
        None,
        description="categories is a list of the grouped resources this resource belongs to (e.g. 'all')",
    )
    group: Optional[str] = Field(
        None,
        description='group is the preferred group of the resource.  Empty implies the group of the containing resource list. For subresources, this may have a different value, for example: Scale".',
    )
    kind: str = Field(
        ...,
        description="kind is the kind for the resource (e.g. 'Foo' is the kind for a resource 'foo')",
    )
    name: str = Field(..., description="name is the plural name of the resource.")
    namespaced: bool = Field(
        ..., description="namespaced indicates if a resource is namespaced or not."
    )
    short_names: Optional[List[str]] = Field(
        None,
        alias="shortNames",
        description="shortNames is a list of suggested short names of the resource.",
    )
    singular_name: str = Field(
        ...,
        alias="singularName",
        description="singularName is the singular name of the resource.  This allows clients to handle plural and singular opaquely. The singularName is more correct for reporting status on a single item and both singular and plural are allowed from the kubectl CLI interface.",
    )
    storage_version_hash: Optional[str] = Field(
        None,
        alias="storageVersionHash",
        description="The hash value of the storage version, the version this resource is converted to when written to the data store. Value must be treated as opaque by clients. Only equality comparison on the value is valid. This is an alpha feature and may change or be removed in the future. The field is populated by the apiserver only if the StorageVersionHash feature gate is enabled. This field will remain optional even if it graduates.",
    )
    verbs: List[str] = Field(
        ...,
        description="verbs is a list of supported kube verbs (this includes get, list, watch, create, update, patch, delete, deletecollection, and proxy)",
    )
    version: Optional[str] = Field(
        None,
        description="version is the preferred version of the resource.  Empty implies the version of the containing resource list For subresources, this may have a different value, for example: v1 (while inside a v1beta1 version of the core resource's group)\".",
    )


class APIResourceList(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        "v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    group_version: str = Field(
        ...,
        alias="groupVersion",
        description="groupVersion is the group and version this APIResourceList is for.",
    )
    kind: Optional[Kind185] = Field(
        "APIResourceList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    resources: List[APIResource] = Field(
        ...,
        description="resources contains the name of the resources and if they are namespaced.",
    )


class FieldsV1(BaseModel):
    pass

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"


class GroupVersionForDiscovery(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    group_version: str = Field(
        ...,
        alias="groupVersion",
        description='groupVersion specifies the API group and version in the form "group/version"',
    )
    version: str = Field(
        ...,
        description='version specifies the version in the form of "version". This is to save the clients the trouble of splitting the GroupVersion.',
    )


class LabelSelectorRequirement(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    key: str = Field(
        ..., description="key is the label key that the selector applies to."
    )
    operator: str = Field(
        ...,
        description="operator represents a key's relationship to a set of values. Valid operators are In, NotIn, Exists and DoesNotExist.",
    )
    values: Optional[List[str]] = Field(
        None,
        description="values is an array of string values. If the operator is In or NotIn, the values array must be non-empty. If the operator is Exists or DoesNotExist, the values array must be empty. This array is replaced during a strategic merge patch.",
    )


class ListMeta(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    continue_: Optional[str] = Field(
        None,
        alias="continue",
        description="continue may be set if the user set a limit on the number of items returned, and indicates that the server has more data available. The value is opaque and may be used to issue another request to the endpoint that served this list to retrieve the next set of available objects. Continuing a consistent list may not be possible if the server configuration has changed or more than a few minutes have passed. The resourceVersion field returned when using this continue value will be identical to the value in the first response, unless you have received this token from an error message.",
    )
    remaining_item_count: Optional[int] = Field(
        None,
        alias="remainingItemCount",
        description="remainingItemCount is the number of subsequent items in the list which are not included in this list response. If the list request contained label or field selectors, then the number of remaining items is unknown and the field will be left unset and omitted during serialization. If the list is complete (either because it is not chunking or because this is the last chunk), then there are no more remaining items and this field will be left unset and omitted during serialization. Servers older than v1.15 do not set this field. The intended use of the remainingItemCount is *estimating* the size of a collection. Clients should not rely on the remainingItemCount to be set or to be exact.",
    )
    resource_version: Optional[str] = Field(
        None,
        alias="resourceVersion",
        description="String that identifies the server's internal version of this object that can be used by clients to determine when objects have changed. Value must be treated as opaque by clients and passed unmodified back to the server. Populated by the system. Read-only. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#concurrency-control-and-consistency",
    )
    self_link: Optional[str] = Field(
        None,
        alias="selfLink",
        description="selfLink is a URL representing this object. Populated by the system. Read-only.\n\nDEPRECATED Kubernetes will stop propagating this field in 1.20 release and the field is planned to be removed in 1.21 release.",
    )


class MicroTime(BaseModel):
    __root__: datetime


class OwnerReference(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: str = Field(
        ..., alias="apiVersion", description="API version of the referent."
    )
    block_owner_deletion: Optional[bool] = Field(
        None,
        alias="blockOwnerDeletion",
        description='If true, AND if the owner has the "foregroundDeletion" finalizer, then the owner cannot be deleted from the key-value store until this reference is removed. Defaults to false. To set this field, a user needs "delete" permission of the owner, otherwise 422 (Unprocessable Entity) will be returned.',
    )
    controller: Optional[bool] = Field(
        None, description="If true, this reference points to the managing controller."
    )
    kind: str = Field(
        ...,
        description="Kind of the referent. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    name: str = Field(
        ...,
        description="Name of the referent. More info: http://kubernetes.io/docs/user-guide/identifiers#names",
    )
    uid: str = Field(
        ...,
        description="UID of the referent. More info: http://kubernetes.io/docs/user-guide/identifiers#uids",
    )


class Patch(BaseModel):
    pass

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"


class Preconditions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    resource_version: Optional[str] = Field(
        None,
        alias="resourceVersion",
        description="Specifies the target ResourceVersion",
    )
    uid: Optional[str] = Field(None, description="Specifies the target UID.")


class ServerAddressByClientCIDR(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    client_cidr: str = Field(
        ...,
        alias="clientCIDR",
        description="The CIDR with which clients can match their IP to figure out the server address that they should use.",
    )
    server_address: str = Field(
        ...,
        alias="serverAddress",
        description="Address of this server, suitable for a client that matches the above CIDR. This can be a hostname, hostname:port, IP or IP:port.",
    )


class StatusCause(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    field: Optional[str] = Field(
        None,
        description='The field of the resource that has caused this error, as named by its JSON serialization. May include dot and postfix notation for nested attributes. Arrays are zero-indexed.  Fields may appear more than once in an array of causes due to fields having multiple errors. Optional.\n\nExamples:\n  "name" - the field "name" on the current resource\n  "items[0].name" - the field "name" on the first array entry in "items"',
    )
    message: Optional[str] = Field(
        None,
        description="A human-readable description of the cause of the error.  This field may be presented as-is to a reader.",
    )
    reason: Optional[str] = Field(
        None,
        description="A machine-readable description of the cause of the error. If this value is empty there is no information available.",
    )


class StatusDetails(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    causes: Optional[List[StatusCause]] = Field(
        None,
        description="The Causes array includes more details associated with the StatusReason failure. Not all StatusReasons may provide detailed causes.",
    )
    group: Optional[str] = Field(
        None,
        description="The group attribute of the resource associated with the status StatusReason.",
    )
    kind: Optional[str] = Field(
        None,
        description="The kind attribute of the resource associated with the status StatusReason. On some operations may differ from the requested resource Kind. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    name: Optional[str] = Field(
        None,
        description="The name attribute of the resource associated with the status StatusReason (when there is a single name which can be described).",
    )
    retry_after_seconds: Optional[int] = Field(
        None,
        alias="retryAfterSeconds",
        description="If specified, the time in seconds before the operation should be retried. Some errors may indicate the client must take an alternate action - for those errors this field may indicate how long to wait before taking the alternate action.",
    )
    uid: Optional[str] = Field(
        None,
        description="UID of the resource. (when there is a single resource which can be described). More info: http://kubernetes.io/docs/user-guide/identifiers#uids",
    )


class Time(BaseModel):
    __root__: datetime


class APIGroup(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        "v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind183] = Field(
        "APIGroup",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    name: str = Field(..., description="name is the name of the group.")
    preferred_version: Optional[GroupVersionForDiscovery] = Field(
        None,
        alias="preferredVersion",
        description="preferredVersion is the version preferred by the API server, which probably is the storage version.",
    )
    server_address_by_client_cid_rs: Optional[List[ServerAddressByClientCIDR]] = Field(
        None,
        alias="serverAddressByClientCIDRs",
        description="a map of client CIDR to server address that is serving this group. This is to help clients reach servers in the most network-efficient way possible. Clients can use the appropriate server address as per the CIDR that they match. In case of multiple matches, clients should use the longest matching CIDR. The server returns only those CIDRs that it thinks that the client can match. For example: the master will return an internal IP CIDR only, if the client reaches the server using an internal IP. Server looks at X-Forwarded-For header or X-Real-Ip header or request.RemoteAddr (in that order) to get the client IP.",
    )
    versions: List[GroupVersionForDiscovery] = Field(
        ..., description="versions are the versions supported in this group."
    )


class APIGroupList(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        "v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    groups: List[APIGroup] = Field(..., description="groups is a list of APIGroup.")
    kind: Optional[Kind184] = Field(
        "APIGroupList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )


class APIVersions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        "v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind186] = Field(
        "APIVersions",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    server_address_by_client_cid_rs: List[ServerAddressByClientCIDR] = Field(
        ...,
        alias="serverAddressByClientCIDRs",
        description="a map of client CIDR to server address that is serving this group. This is to help clients reach servers in the most network-efficient way possible. Clients can use the appropriate server address as per the CIDR that they match. In case of multiple matches, clients should use the longest matching CIDR. The server returns only those CIDRs that it thinks that the client can match. For example: the master will return an internal IP CIDR only, if the client reaches the server using an internal IP. Server looks at X-Forwarded-For header or X-Real-Ip header or request.RemoteAddr (in that order) to get the client IP.",
    )
    versions: List[str] = Field(
        ..., description="versions are the api versions that are available."
    )


class DeleteOptions(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        "v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    dry_run: Optional[List[str]] = Field(
        None,
        alias="dryRun",
        description="When present, indicates that modifications should not be persisted. An invalid or unrecognized dryRun directive will result in an error response and no further processing of the request. Valid values are: - All: all dry run stages will be processed",
    )
    grace_period_seconds: Optional[int] = Field(
        None,
        alias="gracePeriodSeconds",
        description="The duration in seconds before the object should be deleted. Value must be non-negative integer. The value zero indicates delete immediately. If this value is nil, the default grace period for the specified type will be used. Defaults to a per object value if not specified. zero means delete immediately.",
    )
    kind: Optional[Kind187] = Field(
        "DeleteOptions",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    orphan_dependents: Optional[bool] = Field(
        None,
        alias="orphanDependents",
        description='Deprecated: please use the PropagationPolicy, this field will be deprecated in 1.7. Should the dependent objects be orphaned. If true/false, the "orphan" finalizer will be added to/removed from the object\'s finalizers list. Either this field or PropagationPolicy may be set, but not both.',
    )
    preconditions: Optional[Preconditions] = Field(
        None,
        description="Must be fulfilled before a deletion is carried out. If not possible, a 409 Conflict status will be returned.",
    )
    propagation_policy: Optional[str] = Field(
        None,
        alias="propagationPolicy",
        description="Whether and how garbage collection will be performed. Either this field or OrphanDependents may be set, but not both. The default policy is decided by the existing finalizer set in the metadata.finalizers and the resource-specific default policy. Acceptable values are: 'Orphan' - orphan the dependents; 'Background' - allow the garbage collector to delete the dependents in the background; 'Foreground' - a cascading policy that deletes all dependents in the foreground.",
    )


class LabelSelector(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    match_expressions: Optional[List[LabelSelectorRequirement]] = Field(
        None,
        alias="matchExpressions",
        description="matchExpressions is a list of label selector requirements. The requirements are ANDed.",
    )
    match_labels: Optional[Dict[str, Any]] = Field(
        None,
        alias="matchLabels",
        description='matchLabels is a map of {key,value} pairs. A single {key,value} in the matchLabels map is equivalent to an element of matchExpressions, whose key field is "key", the operator is "In", and the values array contains only "value". The requirements are ANDed.',
    )


class ManagedFieldsEntry(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        None,
        alias="apiVersion",
        description='APIVersion defines the version of this resource that this field set applies to. The format is "group/version" just like the top-level APIVersion field. It is necessary to track the version of a field set because it cannot be automatically converted.',
    )
    fields_type: Optional[str] = Field(
        None,
        alias="fieldsType",
        description='FieldsType is the discriminator for the different fields format and version. There is currently only one possible value: "FieldsV1"',
    )
    fields_v1: Optional[FieldsV1] = Field(
        None,
        alias="fieldsV1",
        description='FieldsV1 holds the first JSON version format as described in the "FieldsV1" type.',
    )
    manager: Optional[str] = Field(
        None,
        description="Manager is an identifier of the workflow managing these fields.",
    )
    operation: Optional[str] = Field(
        None,
        description="Operation is the type of operation which lead to this ManagedFieldsEntry being created. The only valid values for this field are 'Apply' and 'Update'.",
    )
    time: Optional[Time] = Field(
        None,
        description="Time is timestamp of when these fields were set. It should always be empty if Operation is 'Apply'",
    )


class ObjectMeta(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    annotations: Optional[Dict[str, Any]] = Field(
        None,
        description="Annotations is an unstructured key value map stored with a resource that may be set by external tools to store and retrieve arbitrary metadata. They are not queryable and should be preserved when modifying objects. More info: http://kubernetes.io/docs/user-guide/annotations",
    )
    cluster_name: Optional[str] = Field(
        None,
        alias="clusterName",
        description="The name of the cluster which the object belongs to. This is used to distinguish resources with same name and namespace in different clusters. This field is not set anywhere right now and apiserver is going to ignore it if set in create or update request.",
    )
    creation_timestamp: Optional[Time] = Field(
        None,
        alias="creationTimestamp",
        description="CreationTimestamp is a timestamp representing the server time when this object was created. It is not guaranteed to be set in happens-before order across separate operations. Clients may not set this value. It is represented in RFC3339 form and is in UTC.\n\nPopulated by the system. Read-only. Null for lists. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    deletion_grace_period_seconds: Optional[int] = Field(
        None,
        alias="deletionGracePeriodSeconds",
        description="Number of seconds allowed for this object to gracefully terminate before it will be removed from the system. Only set when deletionTimestamp is also set. May only be shortened. Read-only.",
    )
    deletion_timestamp: Optional[Time] = Field(
        None,
        alias="deletionTimestamp",
        description="DeletionTimestamp is RFC 3339 date and time at which this resource will be deleted. This field is set by the server when a graceful deletion is requested by the user, and is not directly settable by a client. The resource is expected to be deleted (no longer visible from resource lists, and not reachable by name) after the time in this field, once the finalizers list is empty. As long as the finalizers list contains items, deletion is blocked. Once the deletionTimestamp is set, this value may not be unset or be set further into the future, although it may be shortened or the resource may be deleted prior to this time. For example, a user may request that a pod is deleted in 30 seconds. The Kubelet will react by sending a graceful termination signal to the containers in the pod. After that 30 seconds, the Kubelet will send a hard termination signal (SIGKILL) to the container and after cleanup, remove the pod from the API. In the presence of network partitions, this object may still exist after this timestamp, until an administrator or automated process can determine the resource is fully terminated. If not set, graceful deletion of the object has not been requested.\n\nPopulated by the system when a graceful deletion is requested. Read-only. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    finalizers: Optional[List[str]] = Field(
        None,
        description="Must be empty before the object is deleted from the registry. Each entry is an identifier for the responsible component that will remove the entry from the list. If the deletionTimestamp of the object is non-nil, entries in this list can only be removed.",
    )
    generate_name: Optional[str] = Field(
        None,
        alias="generateName",
        description="GenerateName is an optional prefix, used by the server, to generate a unique name ONLY IF the Name field has not been provided. If this field is used, the name returned to the client will be different than the name passed. This value will also be combined with a unique suffix. The provided value has the same validation rules as the Name field, and may be truncated by the length of the suffix required to make the value unique on the server.\n\nIf this field is specified and the generated name exists, the server will NOT return a 409 - instead, it will either return 201 Created or 500 with Reason ServerTimeout indicating a unique name could not be found in the time allotted, and the client should retry (optionally after the time indicated in the Retry-After header).\n\nApplied only if Name is not specified. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#idempotency",
    )
    generation: Optional[int] = Field(
        None,
        description="A sequence number representing a specific generation of the desired state. Populated by the system. Read-only.",
    )
    labels: Optional[Dict[str, Any]] = Field(
        None,
        description="Map of string keys and values that can be used to organize and categorize (scope and select) objects. May match selectors of replication controllers and services. More info: http://kubernetes.io/docs/user-guide/labels",
    )
    managed_fields: Optional[List[ManagedFieldsEntry]] = Field(
        None,
        alias="managedFields",
        description="ManagedFields maps workflow-id and version to the set of fields that are managed by that workflow. This is mostly for internal housekeeping, and users typically shouldn't need to set or understand this field. A workflow can be the user's name, a controller's name, or the name of a specific apply path like \"ci-cd\". The set of fields is always in the version that the workflow used when modifying the object.",
    )
    name: Optional[str] = Field(
        None,
        description="Name must be unique within a namespace. Is required when creating resources, although some resources may allow a client to request the generation of an appropriate name automatically. Name is primarily intended for creation idempotence and configuration definition. Cannot be updated. More info: http://kubernetes.io/docs/user-guide/identifiers#names",
    )
    namespace: Optional[str] = Field(
        None,
        description='Namespace defines the space within each name must be unique. An empty namespace is equivalent to the "default" namespace, but "default" is the canonical representation. Not all objects are required to be scoped to a namespace - the value of this field for those objects will be empty.\n\nMust be a DNS_LABEL. Cannot be updated. More info: http://kubernetes.io/docs/user-guide/namespaces',
    )
    owner_references: Optional[List[OwnerReference]] = Field(
        None,
        alias="ownerReferences",
        description="List of objects depended by this object. If ALL objects in the list have been deleted, this object will be garbage collected. If this object is managed by a controller, then an entry in this list will point to this controller, with the controller field set to true. There cannot be more than one managing controller.",
    )
    resource_version: Optional[str] = Field(
        None,
        alias="resourceVersion",
        description="An opaque value that represents the internal version of this object that can be used by clients to determine when objects have changed. May be used for optimistic concurrency, change detection, and the watch operation on a resource or set of resources. Clients must treat these values as opaque and passed unmodified back to the server. They may only be valid for a particular resource or set of resources.\n\nPopulated by the system. Read-only. Value must be treated as opaque by clients and . More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#concurrency-control-and-consistency",
    )
    self_link: Optional[str] = Field(
        None,
        alias="selfLink",
        description="SelfLink is a URL representing this object. Populated by the system. Read-only.\n\nDEPRECATED Kubernetes will stop propagating this field in 1.20 release and the field is planned to be removed in 1.21 release.",
    )
    uid: Optional[str] = Field(
        None,
        description="UID is the unique in time and space value for this object. It is typically generated by the server on successful creation of a resource and is not allowed to change on PUT operations.\n\nPopulated by the system. Read-only. More info: http://kubernetes.io/docs/user-guide/identifiers#uids",
    )


class Status(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    code: Optional[int] = Field(
        None, description="Suggested HTTP return code for this status, 0 if not set."
    )
    details: Optional[StatusDetails] = Field(
        None,
        description="Extended data associated with the reason.  Each reason may define its own extended details. This field is optional and the data returned is not guaranteed to conform to any schema except that defined by the reason type.",
    )
    kind: Optional[Kind188] = Field(
        "Status",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    message: Optional[str] = Field(
        None,
        description="A human-readable description of the status of this operation.",
    )
    metadata: Optional[ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    reason: Optional[str] = Field(
        None,
        description='A machine-readable description of why this operation is in the "Failure" status. If this value is empty there is no information available. A Reason clarifies an HTTP status code but does not override it.',
    )


class WatchEvent(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    object: runtime.RawExtension = Field(
        ...,
        description="Object is:\n * If Type is Added or Modified: the new state of the object.\n * If Type is Deleted: the state of the object immediately before deletion.\n * If Type is Error: *Status is recommended; other types may make sense\n   depending on context.",
    )
    type: str



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from pydantic import BaseModel


class RawExtension(BaseModel):
    pass

    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import Any, Dict, List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind63, Kind64
from ...apimachinery.pkg.apis.meta import v1


class CertificateSigningRequestSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    extra: Optional[Dict[str, Any]] = Field(
        None,
        description="Extra information about the requesting user. See user.Info interface for details.",
    )
    groups: Optional[List[str]] = Field(
        None,
        description="Group information about the requesting user. See user.Info interface for details.",
    )
    request: str = Field(..., description="Base64-encoded PKCS#10 CSR data")
    uid: Optional[str] = Field(
        None,
        description="UID information about the requesting user. See user.Info interface for details.",
    )
    usages: Optional[List[str]] = Field(
        None,
        description="allowedUsages specifies a set of usage contexts the key will be valid for. See: https://tools.ietf.org/html/rfc5280#section-4.2.1.3\n     https://tools.ietf.org/html/rfc5280#section-4.2.1.12",
    )
    username: Optional[str] = Field(
        None,
        description="Information about the requesting user. See user.Info interface for details.",
    )


class CertificateSigningRequestCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_update_time: Optional[v1.Time] = Field(
        None,
        alias="lastUpdateTime",
        description="timestamp for the last update to this condition",
    )
    message: Optional[str] = Field(
        None, description="human readable message with details about the request state"
    )
    reason: Optional[str] = Field(
        None, description="brief reason for the request state"
    )
    type: str = Field(
        ..., description="request approval state, currently Approved or Denied."
    )


class CertificateSigningRequestStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    certificate: Optional[str] = Field(
        None,
        description="If request was approved, the controller will place the issued certificate here.",
    )
    conditions: Optional[List[CertificateSigningRequestCondition]] = Field(
        None,
        description="Conditions applied to the request, such as approval or denial.",
    )


class CertificateSigningRequest(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "certificates.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind63] = Field(
        "CertificateSigningRequest",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: Optional[CertificateSigningRequestSpec] = Field(
        None,
        description="The certificate request itself and any additional information.",
    )


class CertificateSigningRequestList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "certificates.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[CertificateSigningRequest]
    kind: Optional[Kind64] = Field(
        "CertificateSigningRequestList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = None



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind59, Kind60
from ...apimachinery.pkg.apis.meta import v1 as v1_1
from ..core import v1
from . import v1 as v1_1


class CronJobStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    active: Optional[List[v1.ObjectReference]] = Field(
        None, description="A list of pointers to currently running jobs."
    )
    last_schedule_time: Optional[v1_1.Time] = Field(
        None,
        alias="lastScheduleTime",
        description="Information when was the last time the job was successfully scheduled.",
    )


class JobTemplateSpec(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata of the jobs created from this template. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[v1_1.JobSpec] = Field(
        None,
        description="Specification of the desired behavior of the job. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class CronJobSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    concurrency_policy: Optional[str] = Field(
        None,
        alias="concurrencyPolicy",
        description='Specifies how to treat concurrent executions of a Job. Valid values are: - "Allow" (default): allows CronJobs to run concurrently; - "Forbid": forbids concurrent runs, skipping next run if previous run hasn\'t finished yet; - "Replace": cancels currently running job and replaces it with a new one',
    )
    failed_jobs_history_limit: Optional[int] = Field(
        None,
        alias="failedJobsHistoryLimit",
        description="The number of failed finished jobs to retain. This is a pointer to distinguish between explicit zero and not specified. Defaults to 1.",
    )
    job_template: JobTemplateSpec = Field(
        ...,
        alias="jobTemplate",
        description="Specifies the job that will be created when executing a CronJob.",
    )
    schedule: str = Field(
        ...,
        description="The schedule in Cron format, see https://en.wikipedia.org/wiki/Cron.",
    )
    starting_deadline_seconds: Optional[int] = Field(
        None,
        alias="startingDeadlineSeconds",
        description="Optional deadline in seconds for starting the job if it misses scheduled time for any reason.  Missed jobs executions will be counted as failed ones.",
    )
    successful_jobs_history_limit: Optional[int] = Field(
        None,
        alias="successfulJobsHistoryLimit",
        description="The number of successful finished jobs to retain. This is a pointer to distinguish between explicit zero and not specified. Defaults to 3.",
    )
    suspend: Optional[bool] = Field(
        None,
        description="This flag tells the controller to suspend subsequent executions, it does not apply to already started executions.  Defaults to false.",
    )


class CronJob(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "batch/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind59] = Field(
        "CronJob",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object's metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[CronJobSpec] = Field(
        None,
        description="Specification of the desired behavior of a cron job, including the schedule. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status",
    )


class CronJobList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "batch/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[CronJob] = Field(..., description="items is the list of CronJobs.")
    kind: Optional[Kind60] = Field(
        "CronJobList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(
        None,
        description="Standard list metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ....... import Kind191, Kind192
from .....apimachinery.pkg.apis.meta import v1


class APIServiceCondition(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    last_transition_time: Optional[v1.Time] = Field(
        None,
        alias="lastTransitionTime",
        description="Last time the condition transitioned from one status to another.",
    )
    message: Optional[str] = Field(
        None,
        description="Human-readable message indicating details about last transition.",
    )
    reason: Optional[str] = Field(
        None,
        description="Unique, one-word, CamelCase reason for the condition's last transition.",
    )
    type: str = Field(..., description="Type is the type of the condition.")


class APIServiceStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    conditions: Optional[List[APIServiceCondition]] = Field(
        None, description="Current service state of apiService."
    )


class ServiceReference(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    name: Optional[str] = Field(None, description="Name is the name of the service")
    namespace: Optional[str] = Field(
        None, description="Namespace is the namespace of the service"
    )
    port: Optional[int] = Field(
        None,
        description="If specified, the port on the service that hosting webhook. Default to 443 for backward compatibility. `port` should be a valid port number (1-65535, inclusive).",
    )


class APIServiceSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    ca_bundle: Optional[str] = Field(
        None,
        alias="caBundle",
        description="CABundle is a PEM encoded CA bundle which will be used to validate an API server's serving certificate. If unspecified, system trust roots on the apiserver are used.",
    )
    group: Optional[str] = Field(
        None, description="Group is the API group name this server hosts"
    )
    group_priority_minimum: int = Field(
        ...,
        alias="groupPriorityMinimum",
        description="GroupPriorityMininum is the priority this group should have at least. Higher priority means that the group is preferred by clients over lower priority ones. Note that other versions of this group might specify even higher GroupPriorityMininum values such that the whole group gets a higher priority. The primary sort is based on GroupPriorityMinimum, ordered highest number to lowest (20 before 10). The secondary sort is based on the alphabetical comparison of the name of the object.  (v1.bar before v1.foo) We'd recommend something like: *.k8s.io (except extensions) at 18000 and PaaSes (OpenShift, Deis) are recommended to be in the 2000s",
    )
    insecure_skip_tls_verify: Optional[bool] = Field(
        None,
        alias="insecureSkipTLSVerify",
        description="InsecureSkipTLSVerify disables TLS certificate verification when communicating with this server. This is strongly discouraged.  You should use the CABundle instead.",
    )
    service: ServiceReference = Field(
        ...,
        description="Service is a reference to the service for this API server.  It must communicate on port 443 If the Service is nil, that means the handling for the API groupversion is handled locally on this server. The call will simply delegate to the normal handler chain to be fulfilled.",
    )
    version: Optional[str] = Field(
        None,
        description='Version is the API version this server hosts.  For example, "v1"',
    )
    version_priority: int = Field(
        ...,
        alias="versionPriority",
        description='VersionPriority controls the ordering of this API version inside of its group.  Must be greater than zero. The primary sort is based on VersionPriority, ordered highest to lowest (20 before 10). Since it\'s inside of a group, the number can be small, probably in the 10s. In case of equal version priorities, the version string will be used to compute the order inside a group. If the version string is "kube-like", it will sort above non "kube-like" version strings, which are ordered lexicographically. "Kube-like" versions start with a "v", then are followed by a number (the major version), then optionally the string "alpha" or "beta" and another number (the minor version). These are sorted first by GA > beta > alpha (where GA is a version with no suffix such as beta or alpha), and then by comparing major version, then minor version. An example sorted list of versions: v10, v2, v1, v11beta2, v10beta3, v3beta1, v12alpha1, v11alpha2, foo1, foo10.',
    )


class APIService(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apiregistration.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind191] = Field(
        "APIService",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = None
    spec: Optional[APIServiceSpec] = Field(
        None,
        description="Spec contains information for locating and communicating with a server",
    )


class APIServiceList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "apiregistration.k8s.io/v1beta1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[APIService]
    kind: Optional[Kind192] = Field(
        "APIServiceList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = None



# automatically generated file. DO NOT CHANGE MANUALLY

from __future__ import annotations

from typing import List, Optional

from pydantic import BaseModel, Field

import pdk8s.model

from ..... import Kind50, Kind51, Kind52
from ...apimachinery.pkg.apis.meta import v1


class CrossVersionObjectReference(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    api_version: Optional[str] = Field(
        None, alias="apiVersion", description="API version of the referent"
    )
    kind: str = Field(
        ...,
        description='Kind of the referent; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds"',
    )
    name: str = Field(
        ...,
        description="Name of the referent; More info: http://kubernetes.io/docs/user-guide/identifiers#names",
    )


class HorizontalPodAutoscalerSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    max_replicas: int = Field(
        ...,
        alias="maxReplicas",
        description="upper limit for the number of pods that can be set by the autoscaler; cannot be smaller than MinReplicas.",
    )
    min_replicas: Optional[int] = Field(
        None,
        alias="minReplicas",
        description="minReplicas is the lower limit for the number of replicas to which the autoscaler can scale down.  It defaults to 1 pod.  minReplicas is allowed to be 0 if the alpha feature gate HPAScaleToZero is enabled and at least one Object or External metric is configured.  Scaling is active as long as at least one metric value is available.",
    )
    scale_target_ref: CrossVersionObjectReference = Field(
        ...,
        alias="scaleTargetRef",
        description="reference to scaled resource; horizontal pod autoscaler will learn the current resource consumption and will set the desired number of pods by using its Scale subresource.",
    )
    target_cpu_utilization_percentage: Optional[int] = Field(
        None,
        alias="targetCPUUtilizationPercentage",
        description="target average CPU utilization (represented as a percentage of requested CPU) over all the pods; if not specified the default autoscaling policy will be used.",
    )


class ScaleSpec(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    replicas: Optional[int] = Field(
        None, description="desired number of instances for the scaled object."
    )


class ScaleStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    replicas: int = Field(
        ..., description="actual number of observed instances of the scaled object."
    )
    selector: Optional[str] = Field(
        None,
        description="label query over pods that should match the replicas count. This is same as the label selector but in the string format to avoid introspection by clients. The string will be in the same format as the query-param syntax. More info about label selectors: http://kubernetes.io/docs/user-guide/labels#label-selectors",
    )


class HorizontalPodAutoscalerStatus(BaseModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "forbid"

    current_cpu_utilization_percentage: Optional[int] = Field(
        None,
        alias="currentCPUUtilizationPercentage",
        description="current average CPU utilization over all pods, represented as a percentage of requested CPU, e.g. 70 means that an average pod is using now 70% of its requested CPU.",
    )
    current_replicas: int = Field(
        ...,
        alias="currentReplicas",
        description="current number of replicas of pods managed by this autoscaler.",
    )
    desired_replicas: int = Field(
        ...,
        alias="desiredReplicas",
        description="desired number of replicas of pods managed by this autoscaler.",
    )
    last_scale_time: Optional[v1.Time] = Field(
        None,
        alias="lastScaleTime",
        description="last time the HorizontalPodAutoscaler scaled the number of pods; used by the autoscaler to control how often the number of pods is changed.",
    )
    observed_generation: Optional[int] = Field(
        None,
        alias="observedGeneration",
        description="most recent generation observed by this autoscaler.",
    )


class HorizontalPodAutoscaler(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "autoscaling/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind50] = Field(
        "HorizontalPodAutoscaler",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object metadata. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata",
    )
    spec: Optional[HorizontalPodAutoscalerSpec] = Field(
        None,
        description="behaviour of autoscaler. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.",
    )


class HorizontalPodAutoscalerList(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "autoscaling/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    items: List[HorizontalPodAutoscaler] = Field(
        ..., description="list of horizontal pod autoscaler objects."
    )
    kind: Optional[Kind51] = Field(
        "HorizontalPodAutoscalerList",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ListMeta] = Field(None, description="Standard list metadata.")


class Scale(pdk8s.model.NamedModel):
    class Config:
        allow_population_by_field_name = True
        validate_assignment = True
        extra = "allow"

    api_version: Optional[str] = Field(
        "autoscaling/v1",
        alias="apiVersion",
        description="APIVersion defines the versioned schema of this representation of an object. Servers should convert recognized schemas to the latest internal value, and may reject unrecognized values. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#resources",
    )
    kind: Optional[Kind52] = Field(
        "Scale",
        description="Kind is a string value representing the REST resource this object represents. Servers may infer this from the endpoint the client submits requests to. Cannot be updated. In CamelCase. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#types-kinds",
    )
    metadata: Optional[v1.ObjectMeta] = Field(
        None,
        description="Standard object metadata; More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#metadata.",
    )
    spec: Optional[ScaleSpec] = Field(
        None,
        description="defines the behavior of the scale. More info: https://git.k8s.io/community/contributors/devel/sig-architecture/api-conventions.md#spec-and-status.",
    )
